@@ -9,13 +9,10 @@ use std::thread;
 use std::time::Duration;
 
 use env_proxy::for_url;
-use lazy_static::lazy_static;
 use reqwest::{blocking::Client, Proxy};
 use url::Url;
 
-lazy_static! {
-    static ref SERIALISE_TESTS: Mutex<()> = Mutex::new(());
-}
+static SERIALISE_TESTS: Mutex<()> = Mutex::new(());
 
 fn scrub_env() {
     remove_var("http_proxy");
@@ -75,8 +72,11 @@ fn socks_proxy_request() {
 
     if let Err(e) = res {
         let s = e.source().unwrap();
+        assert!(
+            s.to_string().contains("socks connect error"),
+            "Expected socks connect error, got: {s}",
+        );
         assert_eq!(CALL_COUNT.load(Ordering::SeqCst), 1);
-        assert!(s.to_string().contains("socks connect error"));
     } else {
         panic!("Socks proxy was ignored")
     }
