@@ -32,6 +32,7 @@
 
 #[cfg(unix)]
 mod shell;
+#[cfg(feature = "test")]
 pub(crate) mod test;
 #[cfg(unix)]
 mod unix;
@@ -57,26 +58,34 @@ use anyhow::{anyhow, Context, Result};
 use cfg_if::cfg_if;
 use same_file::Handle;
 
-use super::common::{self, ignorable_error, report_error, Confirm};
-use super::errors::*;
-use super::markdown::md;
-use super::term2;
-use crate::cli::term2::Terminal;
-use crate::dist::dist::{self, Profile, TargetTriple};
-use crate::process;
-use crate::toolchain::{DistributableToolchain, Toolchain};
-use crate::utils::utils;
-use crate::utils::Notification;
-use crate::{Cfg, UpdateStatus};
-use crate::{DUP_TOOLS, TOOLS};
+use crate::currentprocess::terminalsource;
+use crate::{
+    cli::{
+        common::{self, ignorable_error, report_error, Confirm, PackageUpdate},
+        errors::*,
+        markdown::md,
+    },
+    currentprocess::{filesource::StdoutSource, varsource::VarSource},
+    dist::dist::{self, PartialToolchainDesc, Profile, TargetTriple, ToolchainDesc},
+    install::UpdateStatus,
+    process,
+    toolchain::{
+        distributable::DistributableToolchain,
+        names::{MaybeOfficialToolchainName, ResolvableToolchainName, ToolchainName},
+        toolchain::Toolchain,
+    },
+    utils::{utils, Notification},
+    Cfg, DUP_TOOLS, TOOLS,
+};
+
 use os::*;
 pub(crate) use os::{delete_rustup_and_cargo_home, run_update, self_replace};
 #[cfg(windows)]
 pub use windows::complete_windows_uninstall;
 
-pub struct InstallOpts<'a> {
+pub(crate) struct InstallOpts<'a> {
     pub default_host_triple: Option<String>,
-    pub default_toolchain: Option<String>,
+    pub default_toolchain: Option<MaybeOfficialToolchainName>,
     pub profile: String,
     pub no_modify_path: bool,
     pub no_update_toolchain: bool,
@@ -206,18 +215,32 @@ but will not be added automatically."
     };
 }
 
+#[cfg(not(windows))]
+macro_rules! post_install_msg_unix_source_env {
+    () => {
+        r#"To configure your current shell, you need to source
+the corresponding `env` file under {cargo_home}.
+
+This is usually done by running one of the following (note the leading DOT):
+    . "{cargo_home}/env"            # For sh/bash/zsh/ash/dash/pdksh
+    source "{cargo_home}/env.fish"  # For fish
+"#
+    };
+}
+
 #[cfg(not(windows))]
 macro_rules! post_install_msg_unix {
     () => {
-        r#"# Rust is installed now. Great!
+        concat!(
+            r"# Rust is installed now. Great!
 
 To get started you may need to restart your current shell.
 This would reload your `PATH` environment variable to include
 Cargo's bin directory ({cargo_home}/bin).
 
-To configure your current shell, run:
-    source "{cargo_home}/env"
-"#
+",
+            post_install_msg_unix_source_env!(),
+        )
     };
 }
 
@@ -237,14 +260,15 @@ Cargo's bin directory ({cargo_home}\\bin).
 #[cfg(not(windows))]
 macro_rules! post_install_msg_unix_no_modify_path {
     () => {
-        r#"# Rust is installed now. Great!
+        concat!(
+            r"# Rust is installed now. Great!
 
 To get started you need Cargo's bin directory ({cargo_home}/bin) in your `PATH`
 environment variable. This has not been done automatically.
 
-To configure your current shell, run:
-    source "{cargo_home}/env"
-"#
+",
+            post_install_msg_unix_source_env!(),
+        )
     };
 }
 
@@ -359,7 +383,7 @@ pub(crate) fn install(
     #[cfg(unix)]
     do_anti_sudo_check(no_prompt)?;
 
-    let mut term = term2::stdout();
+    let mut term = process().stdout().terminal();
 
     #[cfg(windows)]
     if let Some(plan) = do_msvc_check(&opts) {
@@ -409,10 +433,10 @@ pub(crate) fn install(
         let msg = pre_install_msg(opts.no_modify_path)?;
 
         md(&mut term, msg);
-
+        let mut customized_install = false;
         loop {
             md(&mut term, current_install_opts(&opts));
-            match common::confirm_advanced()? {
+            match common::confirm_advanced(customized_install)? {
                 Confirm::No => {
                     info!("aborting installation");
                     return Ok(utils::ExitCode(0));
@@ -421,6 +445,7 @@ pub(crate) fn install(
                     break;
                 }
                 Confirm::Advanced => {
+                    customized_install = true;
                     opts = customize_install(opts)?;
                 }
             }
@@ -439,7 +464,7 @@ pub(crate) fn install(
         }
         utils::create_rustup_home()?;
         maybe_install_rust(
-            opts.default_toolchain.as_deref(),
+            opts.default_toolchain,
             &opts.profile,
             opts.default_host_triple.as_deref(),
             !opts.no_update_toolchain,
@@ -580,6 +605,8 @@ fn do_pre_install_sanity_checks(no_prompt: bool) -> Result<()> {
 }
 
 fn do_pre_install_options_sanity_checks(opts: &InstallOpts<'_>) -> Result<()> {
+    common::warn_if_host_is_emulated();
+
     // Verify that the installation options are vaguely sane
     (|| {
         let host_triple = opts
@@ -587,17 +614,14 @@ fn do_pre_install_options_sanity_checks(opts: &InstallOpts<'_>) -> Result<()> {
             .as_ref()
             .map(|s| dist::TargetTriple::new(s))
             .unwrap_or_else(TargetTriple::from_host_or_build);
-        let toolchain_to_use = match &opts.default_toolchain {
-            None => "stable",
-            Some(s) if s == "none" => "stable",
-            Some(s) => s,
+        let partial_channel = match &opts.default_toolchain {
+            None | Some(MaybeOfficialToolchainName::None) => {
+                ResolvableToolchainName::try_from("stable")?
+            }
+            Some(MaybeOfficialToolchainName::Some(s)) => s.into(),
         };
-        let partial_channel = dist::PartialToolchainDesc::from_str(toolchain_to_use)?;
-        let resolved = partial_channel.resolve(&host_triple)?.to_string();
-        debug!(
-            "Successfully resolved installation toolchain as: {}",
-            resolved
-        );
+        let resolved = partial_channel.resolve(&host_triple)?;
+        debug!("Successfully resolved installation toolchain as: {resolved}");
         Ok(())
     })()
     .map_err(|e: Box<dyn std::error::Error>| {
@@ -606,7 +630,7 @@ fn do_pre_install_options_sanity_checks(opts: &InstallOpts<'_>) -> Result<()> {
              If you are unsure of suitable values, the 'stable' toolchain is the default.\n\
              Valid host triples look something like: {}",
             e,
-            dist::TargetTriple::from_host_or_build()
+            TargetTriple::from_host_or_build()
         )
     })?;
     Ok(())
@@ -667,8 +691,9 @@ fn current_install_opts(opts: &InstallOpts<'_>) -> String {
             .map(|s| TargetTriple::new(s))
             .unwrap_or_else(TargetTriple::from_host_or_build),
         opts.default_toolchain
-            .as_deref()
-            .unwrap_or("stable (default)"),
+            .as_ref()
+            .map(ToString::to_string)
+            .unwrap_or("stable (default)".into()),
         opts.profile,
         if !opts.no_modify_path { "yes" } else { "no" }
     )
@@ -677,12 +702,12 @@ fn current_install_opts(opts: &InstallOpts<'_>) -> String {
 // Interactive editing of the install options
 fn customize_install(mut opts: InstallOpts<'_>) -> Result<InstallOpts<'_>> {
     writeln!(
-        process().stdout(),
+        process().stdout().lock(),
         "I'm going to ask you the value of each of these installation options.\n\
          You may simply press the Enter key to leave unchanged."
     )?;
 
-    writeln!(process().stdout())?;
+    writeln!(process().stdout().lock())?;
 
     opts.default_host_triple = Some(common::question_str(
         "Default host triple?",
@@ -691,10 +716,14 @@ fn customize_install(mut opts: InstallOpts<'_>) -> Result<InstallOpts<'_>> {
             .unwrap_or_else(|| TargetTriple::from_host_or_build().to_string()),
     )?);
 
-    opts.default_toolchain = Some(common::question_str(
+    opts.default_toolchain = Some(MaybeOfficialToolchainName::try_from(common::question_str(
         "Default toolchain? (stable/beta/nightly/none)",
-        opts.default_toolchain.as_deref().unwrap_or("stable"),
-    )?);
+        &opts
+            .default_toolchain
+            .as_ref()
+            .map(ToString::to_string)
+            .unwrap_or("stable".into()),
+    )?)?);
 
     opts.profile = common::question_str(
         &format!(
@@ -727,7 +756,7 @@ fn install_bins() -> Result<()> {
 
 pub(crate) fn install_proxies() -> Result<()> {
     let bin_path = utils::cargo_home()?.join("bin");
-    let rustup_path = bin_path.join(&format!("rustup{EXE_SUFFIX}"));
+    let rustup_path = bin_path.join(format!("rustup{EXE_SUFFIX}"));
 
     let rustup = Handle::from_path(&rustup_path)?;
 
@@ -805,7 +834,7 @@ pub(crate) fn install_proxies() -> Result<()> {
 }
 
 fn maybe_install_rust(
-    toolchain: Option<&str>,
+    toolchain: Option<MaybeOfficialToolchainName>,
     profile_str: &str,
     default_host_triple: Option<&str>,
     update_existing_toolchain: bool,
@@ -825,29 +854,44 @@ fn maybe_install_rust(
         components,
         targets,
     )?;
-    if let Some(toolchain) = toolchain {
-        if toolchain.exists() {
+    if let Some(ref desc) = toolchain {
+        let status = if Toolchain::exists(&cfg, &desc.into())? {
             warn!("Updating existing toolchain, profile choice will be ignored");
-        }
-        let distributable = DistributableToolchain::new(&toolchain)?;
-        let status = distributable.install_from_dist(true, false, components, targets, None)?;
-        let toolchain_str = toolchain.name().to_owned();
-        toolchain.cfg().set_default(&toolchain_str)?;
-        writeln!(process().stdout())?;
-        common::show_channel_update(toolchain.cfg(), &toolchain_str, Ok(status))?;
+            // If we have a partial install we might not be able to read content here. We could:
+            // - fail and folk have to delete the partially present toolchain to recover
+            // - silently ignore it (and provide inconsistent metadata for reporting the install/update change)
+            // - delete the partial install and start over
+            // For now, we error.
+            let mut toolchain = DistributableToolchain::new(&cfg, desc.clone())?;
+            toolchain.update(components, targets, cfg.get_profile()?)?
+        } else {
+            DistributableToolchain::install(
+                &cfg,
+                desc,
+                components,
+                targets,
+                cfg.get_profile()?,
+                true,
+            )?
+            .0
+        };
+
+        cfg.set_default(Some(&desc.into()))?;
+        writeln!(process().stdout().lock())?;
+        common::show_channel_update(&cfg, PackageUpdate::Toolchain(desc.clone()), Ok(status))?;
     }
     Ok(())
 }
 
-fn _install_selection<'a>(
-    cfg: &'a mut Cfg,
-    toolchain_opt: Option<&str>,
+fn _install_selection(
+    cfg: &mut Cfg,
+    toolchain_opt: Option<MaybeOfficialToolchainName>,
     profile_str: &str,
     default_host_triple: Option<&str>,
     update_existing_toolchain: bool,
     components: &[&str],
     targets: &[&str],
-) -> Result<Option<Toolchain<'a>>> {
+) -> Result<Option<ToolchainDesc>> {
     cfg.set_profile(profile_str)?;
 
     if let Some(default_host_triple) = default_host_triple {
@@ -868,39 +912,54 @@ fn _install_selection<'a>(
     // a toolchain (updating if it's already present) and then if neither of
     // those are true, we have a user who doesn't mind, and already has an
     // install, so we leave their setup alone.
-    Ok(if toolchain_opt == Some("none") {
-        info!("skipping toolchain installation");
-        if !components.is_empty() {
-            warn!(
-                "ignoring requested component{}: {}",
-                if components.len() == 1 { "" } else { "s" },
-                components.join(", ")
-            );
-        }
-        if !targets.is_empty() {
-            warn!(
-                "ignoring requested target{}: {}",
-                if targets.len() == 1 { "" } else { "s" },
-                targets.join(", ")
-            );
-        }
-        writeln!(process().stdout())?;
-        None
-    } else if user_specified_something
-        || (update_existing_toolchain && cfg.find_default()?.is_none())
-    {
-        Some(match toolchain_opt {
-            Some(s) => cfg.get_toolchain(s, false)?,
-            None => match cfg.find_default()? {
-                Some(t) => t,
-                None => cfg.get_toolchain("stable", false)?,
-            },
-        })
-    } else {
-        info!("updating existing rustup installation - leaving toolchains alone");
-        writeln!(process().stdout())?;
-        None
-    })
+    Ok(
+        if matches!(toolchain_opt, Some(MaybeOfficialToolchainName::None)) {
+            info!("skipping toolchain installation");
+            if !components.is_empty() {
+                warn!(
+                    "ignoring requested component{}: {}",
+                    if components.len() == 1 { "" } else { "s" },
+                    components.join(", ")
+                );
+            }
+            if !targets.is_empty() {
+                warn!(
+                    "ignoring requested target{}: {}",
+                    if targets.len() == 1 { "" } else { "s" },
+                    targets.join(", ")
+                );
+            }
+            writeln!(process().stdout().lock())?;
+            None
+        } else if user_specified_something
+            || (update_existing_toolchain && cfg.find_default()?.is_none())
+        {
+            match toolchain_opt {
+                Some(s) => {
+                    let toolchain_name = match s {
+                        MaybeOfficialToolchainName::None => unreachable!(),
+                        MaybeOfficialToolchainName::Some(n) => n,
+                    };
+                    Some(toolchain_name.resolve(&cfg.get_default_host_triple()?)?)
+                }
+                None => match cfg.get_default()? {
+                    // Default is installable
+                    Some(ToolchainName::Official(t)) => Some(t),
+                    // Default is custom, presumably from a prior install. Do nothing.
+                    Some(ToolchainName::Custom(_)) => None,
+                    None => Some(
+                        "stable"
+                            .parse::<PartialToolchainDesc>()?
+                            .resolve(&cfg.get_default_host_triple()?)?,
+                    ),
+                },
+            }
+        } else {
+            info!("updating existing rustup installation - leaving toolchains alone");
+            writeln!(process().stdout().lock())?;
+            None
+        },
+    )
 }
 
 pub(crate) fn uninstall(no_prompt: bool) -> Result<utils::ExitCode> {
@@ -912,14 +971,14 @@ pub(crate) fn uninstall(no_prompt: bool) -> Result<utils::ExitCode> {
 
     let cargo_home = utils::cargo_home()?;
 
-    if !cargo_home.join(&format!("bin/rustup{EXE_SUFFIX}")).exists() {
+    if !cargo_home.join(format!("bin/rustup{EXE_SUFFIX}")).exists() {
         return Err(CLIError::NotSelfInstalled { p: cargo_home }.into());
     }
 
     if !no_prompt {
-        writeln!(process().stdout())?;
+        writeln!(process().stdout().lock())?;
         let msg = format!(pre_uninstall_msg!(), cargo_home = canonical_cargo_home()?);
-        md(&mut term2::stdout(), msg);
+        md(&mut process().stdout().terminal(), msg);
         if !common::confirm("\nContinue? (y/N)", false)? {
             info!("aborting uninstallation");
             return Ok(utils::ExitCode(0));
@@ -1017,6 +1076,8 @@ pub(crate) fn uninstall(no_prompt: bool) -> Result<utils::ExitCode> {
 /// rustup-init is stored in `CARGO_HOME`/bin, and then deleted next
 /// time rustup runs.
 pub(crate) fn update(cfg: &Cfg) -> Result<utils::ExitCode> {
+    common::warn_if_host_is_emulated();
+
     use common::SelfUpdatePermission::*;
     let update_permitted = if NEVER_SELF_UPDATE {
         HardFail
@@ -1047,11 +1108,19 @@ pub(crate) fn update(cfg: &Cfg) -> Result<utils::ExitCode> {
                 }
             };
 
-            let _ = common::show_channel_update(cfg, "rustup", Ok(UpdateStatus::Updated(version)));
+            let _ = common::show_channel_update(
+                cfg,
+                PackageUpdate::Rustup,
+                Ok(UpdateStatus::Updated(version)),
+            );
             return run_update(&setup_path);
         }
         None => {
-            let _ = common::show_channel_update(cfg, "rustup", Ok(UpdateStatus::Unchanged));
+            let _ = common::show_channel_update(
+                cfg,
+                PackageUpdate::Rustup,
+                Ok(UpdateStatus::Unchanged),
+            );
             // Try again in case we emitted "tool `{}` is already installed" last time.
             install_proxies()?
         }
@@ -1071,12 +1140,10 @@ fn get_new_rustup_version(path: &Path) -> Option<String> {
 }
 
 fn parse_new_rustup_version(version: String) -> String {
-    use lazy_static::lazy_static;
+    use once_cell::sync::Lazy;
     use regex::Regex;
 
-    lazy_static! {
-        static ref RE: Regex = Regex::new(r"\d+.\d+.\d+[0-9a-zA-Z-]*").unwrap();
-    }
+    static RE: Lazy<Regex> = Lazy::new(|| Regex::new(r"\d+.\d+.\d+[0-9a-zA-Z-]*").unwrap());
 
     let capture = RE.captures(&version);
     let matched_version = match capture {
@@ -1185,31 +1252,32 @@ pub(crate) fn get_available_rustup_version() -> Result<String> {
 }
 
 pub(crate) fn check_rustup_update() -> Result<()> {
-    let mut t = term2::stdout();
+    let mut t = process().stdout().terminal();
     // Get current rustup version
     let current_version = env!("CARGO_PKG_VERSION");
 
     // Get available rustup version
     let available_version = get_available_rustup_version()?;
 
-    let _ = t.attr(term2::Attr::Bold);
-    write!(t, "rustup - ")?;
+    let _ = t.attr(terminalsource::Attr::Bold);
+    write!(t.lock(), "rustup - ")?;
 
     if current_version != available_version {
-        let _ = t.fg(term2::color::YELLOW);
-        write!(t, "Update available")?;
+        let _ = t.fg(terminalsource::Color::Yellow);
+        write!(t.lock(), "Update available")?;
         let _ = t.reset();
-        writeln!(t, " : {current_version} -> {available_version}")?;
+        writeln!(t.lock(), " : {current_version} -> {available_version}")?;
     } else {
-        let _ = t.fg(term2::color::GREEN);
-        write!(t, "Up to date")?;
+        let _ = t.fg(terminalsource::Color::Green);
+        write!(t.lock(), "Up to date")?;
         let _ = t.reset();
-        writeln!(t, " : {current_version}")?;
+        writeln!(t.lock(), " : {current_version}")?;
     }
 
     Ok(())
 }
 
+#[cfg_attr(feature = "otel", tracing::instrument)]
 pub(crate) fn cleanup_self_updater() -> Result<()> {
     let cargo_home = utils::cargo_home()?;
     let setup = cargo_home.join(format!("bin/rustup-init{EXE_SUFFIX}"));
@@ -1235,8 +1303,10 @@ mod tests {
 
     use anyhow::Result;
 
+    use rustup_macros::unit_test as test;
+
     use crate::cli::common;
-    use crate::dist::dist::ToolchainDesc;
+    use crate::dist::dist::PartialToolchainDesc;
     use crate::test::{test_dir, with_rustup_home, Env};
     use crate::{currentprocess, for_host};
 
@@ -1245,16 +1315,20 @@ mod tests {
         with_rustup_home(|home| {
             let mut vars = HashMap::new();
             home.apply(&mut vars);
-            let tp = Box::new(currentprocess::TestProcess {
+            let tp = currentprocess::TestProcess {
                 vars,
                 ..Default::default()
-            });
-            currentprocess::with(tp.clone(), || -> Result<()> {
+            };
+            currentprocess::with(tp.clone().into(), || -> Result<()> {
                 // TODO: we could pass in a custom cfg to get notification
                 // callbacks rather than output to the tp sink.
                 let mut cfg = common::set_globals(false, false).unwrap();
                 assert_eq!(
-                    "stable",
+                    "stable"
+                        .parse::<PartialToolchainDesc>()
+                        .unwrap()
+                        .resolve(&cfg.get_default_host_triple().unwrap())
+                        .unwrap(),
                     super::_install_selection(
                         &mut cfg,
                         None,      // No toolchain specified
@@ -1266,10 +1340,6 @@ mod tests {
                     )
                     .unwrap() // result
                     .unwrap() // option
-                    .name()
-                    .parse::<ToolchainDesc>()
-                    .unwrap()
-                    .channel
                 );
                 Ok(())
             })?;
@@ -1292,11 +1362,11 @@ info: default host triple is {0}
         let cargo_home = root_dir.path().join("cargo");
         let mut vars = HashMap::new();
         vars.env("CARGO_HOME", cargo_home.to_string_lossy().to_string());
-        let tp = Box::new(currentprocess::TestProcess {
+        let tp = currentprocess::TestProcess {
             vars,
             ..Default::default()
-        });
-        currentprocess::with(tp, || -> Result<()> {
+        };
+        currentprocess::with(tp.into(), || -> Result<()> {
             super::install_bins().unwrap();
             Ok(())
         })
