@@ -29,7 +29,7 @@ use std::path::PathBuf;
 use anyhow::{bail, Result};
 
 use super::utils;
-use crate::process;
+use crate::{currentprocess::varsource::VarSource, process};
 
 pub(crate) type Shell = Box<dyn UnixShell>;
 
@@ -71,7 +71,12 @@ pub(crate) fn cargo_home_str() -> Result<Cow<'static, str>> {
 // TODO?: Make a decision on Ion Shell, Power Shell, Nushell
 // Cross-platform non-POSIX shells have not been assessed for integration yet
 fn enumerate_shells() -> Vec<Shell> {
-    vec![Box::new(Posix), Box::new(Bash), Box::new(Zsh)]
+    vec![
+        Box::new(Posix),
+        Box::new(Bash),
+        Box::new(Zsh),
+        Box::new(Fish),
+    ]
 }
 
 pub(crate) fn get_available_shells() -> impl Iterator<Item = Shell> {
@@ -161,7 +166,7 @@ impl Zsh {
             }
         } else {
             match std::process::Command::new("zsh")
-                .args(["-c", "'echo $ZDOTDIR'"])
+                .args(["-c", "echo -n $ZDOTDIR"])
                 .output()
             {
                 Ok(io) if !io.stdout.is_empty() => Ok(PathBuf::from(OsStr::from_bytes(&io.stdout))),
@@ -175,7 +180,7 @@ impl UnixShell for Zsh {
     fn does_exist(&self) -> bool {
         // zsh has to either be the shell or be callable for zsh setup.
         matches!(process().var("SHELL"), Ok(sh) if sh.contains("zsh"))
-            || matches!(utils::find_cmd(&["zsh"]), Some(_))
+            || utils::find_cmd(&["zsh"]).is_some()
     }
 
     fn rcfiles(&self) -> Vec<PathBuf> {
@@ -195,12 +200,54 @@ impl UnixShell for Zsh {
         self.rcfiles()
             .into_iter()
             .filter(|env| env.is_file())
-            .chain(self.rcfiles().into_iter())
+            .chain(self.rcfiles())
             .take(1)
             .collect()
     }
 }
 
+struct Fish;
+
+impl UnixShell for Fish {
+    fn does_exist(&self) -> bool {
+        // fish has to either be the shell or be callable for fish setup.
+        matches!(process().var("SHELL"), Ok(sh) if sh.contains("fish"))
+            || utils::find_cmd(&["fish"]).is_some()
+    }
+
+    // > "$XDG_CONFIG_HOME/fish/conf.d" (or "~/.config/fish/conf.d" if that variable is unset) for the user
+    // from <https://github.com/fish-shell/fish-shell/issues/3170#issuecomment-228311857>
+    fn rcfiles(&self) -> Vec<PathBuf> {
+        let p0 = process().var("XDG_CONFIG_HOME").ok().map(|p| {
+            let mut path = PathBuf::from(p);
+            path.push("fish/conf.d/rustup.fish");
+            path
+        });
+
+        let p1 = utils::home_dir().map(|mut path| {
+            path.push(".config/fish/conf.d/rustup.fish");
+            path
+        });
+
+        p0.into_iter().chain(p1).collect()
+    }
+
+    fn update_rcs(&self) -> Vec<PathBuf> {
+        self.rcfiles()
+    }
+
+    fn env_script(&self) -> ShellScript {
+        ShellScript {
+            name: "env.fish",
+            content: include_str!("env.fish"),
+        }
+    }
+
+    fn source_string(&self) -> Result<String> {
+        Ok(format!(r#". "{}/env.fish""#, cargo_home_str()?))
+    }
+}
+
 pub(crate) fn legacy_paths() -> impl Iterator<Item = PathBuf> {
     let zprofiles = Zsh::zdotdir()
         .into_iter()
