@@ -5,6 +5,7 @@ use anyhow::{bail, Context, Result};
 
 use super::install_bins;
 use super::shell;
+use crate::currentprocess::varsource::VarSource;
 use crate::process;
 use crate::utils::utils;
 use crate::utils::Notification;
@@ -140,7 +141,7 @@ pub(crate) fn run_update(setup_path: &Path) -> Result<utils::ExitCode> {
 }
 
 /// This function is as the final step of a self-upgrade. It replaces
-/// `CARGO_HOME`/bin/rustup with the running exe, and updates the the
+/// `CARGO_HOME`/bin/rustup with the running exe, and updates the
 /// links to it.
 pub(crate) fn self_replace() -> Result<utils::ExitCode> {
     install_bins()?;
