@@ -60,11 +60,13 @@ use std::io::{self, Write};
 use std::ops::{Deref, DerefMut};
 use std::path::{Path, PathBuf};
 use std::sync::mpsc::Receiver;
+use std::thread::available_parallelism;
 use std::time::{Duration, Instant};
 use std::{fmt::Debug, fs::OpenOptions};
 
 use anyhow::{Context, Result};
 
+use crate::currentprocess::varsource::VarSource;
 use crate::process;
 use crate::utils::notifications::Notification;
 use threaded::PoolReference;
@@ -451,7 +453,7 @@ pub(crate) fn get_executor<'a>(
 ) -> Result<Box<dyn Executor + 'a>> {
     // If this gets lots of use, consider exposing via the config file.
     let thread_count = match process().var("RUSTUP_IO_THREADS") {
-        Err(_) => num_cpus::get(),
+        Err(_) => available_parallelism().map(|p| p.get()).unwrap_or(1),
         Ok(n) => n
             .parse::<usize>()
             .context("invalid value in RUSTUP_IO_THREADS. Must be a natural number")?,
