@@ -1,4 +1,4 @@
-// Copied from rustc. isatty crate did not work as expected
+// Copied from rustc. isatty crate did not work as expected.
 #[cfg(unix)]
 pub(crate) fn stderr_isatty() -> bool {
     isatty(libc::STDERR_FILENO)
@@ -28,11 +28,6 @@ fn isatty(fd: libc::c_int) -> bool {
 #[inline]
 #[cfg(windows)]
 fn isatty(fd: winapi::shared::minwindef::DWORD) -> bool {
-    if std::env::var("MSYSTEM").is_ok() {
-        // FIXME: No color is better than broken color codes in MSYS shells
-        //        https://github.com/rust-lang/rustup/issues/2292
-        return false;
-    }
     use winapi::um::{consoleapi::GetConsoleMode, processenv::GetStdHandle};
     unsafe {
         let handle = GetStdHandle(fd);
