@@ -1,25 +1,26 @@
-use std::cmp::Ord;
 use std::env;
 use std::fs::{self, File};
 use std::io::{self, BufReader, Write};
 use std::path::{Path, PathBuf};
-use std::process::Command;
 
 use anyhow::{anyhow, bail, Context, Result};
+use home::env as home;
 use retry::delay::{jitter, Fibonacci};
 use retry::{retry, OperationResult};
 use sha2::Sha256;
 use url::Url;
 
-// use crate::currentprocess::cwdsource::CurrentDirSource;
+use crate::currentprocess::{cwdsource::CurrentDirSource, varsource::VarSource};
 use crate::errors::*;
 use crate::utils::notifications::Notification;
 use crate::utils::raw;
 use crate::{home_process, process};
 
-pub use crate::utils::utils::raw::{
-    find_cmd, has_cmd, if_not_empty, is_directory, is_file, path_exists, prefix_arg, random_string,
-};
+#[cfg(not(windows))]
+pub(crate) use crate::utils::utils::raw::find_cmd;
+pub(crate) use crate::utils::utils::raw::{if_not_empty, is_directory};
+
+pub use crate::utils::utils::raw::{is_file, path_exists};
 
 pub struct ExitCode(pub i32);
 
@@ -40,13 +41,6 @@ where
     })
 }
 
-pub fn open_file(name: &'static str, path: &Path) -> Result<File> {
-    File::open(path).with_context(|| RustupError::ReadingFile {
-        name,
-        path: PathBuf::from(path),
-    })
-}
-
 pub fn read_file(name: &'static str, path: &Path) -> Result<String> {
     fs::read_to_string(path).with_context(|| RustupError::ReadingFile {
         name,
@@ -61,22 +55,27 @@ pub fn write_file(name: &'static str, path: &Path, contents: &str) -> Result<()>
     })
 }
 
-pub fn append_file(name: &'static str, path: &Path, line: &str) -> Result<()> {
+pub(crate) fn append_file(name: &'static str, path: &Path, line: &str) -> Result<()> {
     raw::append_file(path, line).with_context(|| RustupError::WritingFile {
         name,
         path: PathBuf::from(path),
     })
 }
 
-pub fn write_line(name: &'static str, file: &mut File, path: &Path, line: &str) -> Result<()> {
-    writeln!(file, "{}", line).with_context(|| RustupError::WritingFile {
+pub(crate) fn write_line(
+    name: &'static str,
+    mut file: impl Write,
+    path: &Path,
+    line: &str,
+) -> Result<()> {
+    writeln!(file, "{line}").with_context(|| RustupError::WritingFile {
         name,
         path: path.to_path_buf(),
     })
 }
 
-pub fn write_str(name: &'static str, file: &mut File, path: &Path, s: &str) -> Result<()> {
-    write!(file, "{}", s).with_context(|| RustupError::WritingFile {
+pub(crate) fn write_str(name: &'static str, file: &mut File, path: &Path, s: &str) -> Result<()> {
+    write!(file, "{s}").with_context(|| RustupError::WritingFile {
         name,
         path: path.to_path_buf(),
     })
@@ -94,7 +93,7 @@ where
     rename(name, src, dest, notify)
 }
 
-pub fn rename_dir<'a, N>(
+pub(crate) fn rename_dir<'a, N>(
     name: &'static str,
     src: &'a Path,
     dest: &'a Path,
@@ -106,7 +105,7 @@ where
     rename(name, src, dest, notify)
 }
 
-pub fn filter_file<F: FnMut(&str) -> bool>(
+pub(crate) fn filter_file<F: FnMut(&str) -> bool>(
     name: &'static str,
     src: &Path,
     dest: &Path,
@@ -122,7 +121,7 @@ pub fn filter_file<F: FnMut(&str) -> bool>(
     })
 }
 
-pub fn canonicalize_path<'a, N>(path: &'a Path, notify_handler: &dyn Fn(N)) -> PathBuf
+pub(crate) fn canonicalize_path<'a, N>(path: &'a Path, notify_handler: &dyn Fn(N)) -> PathBuf
 where
     N: From<Notification<'a>>,
 {
@@ -138,10 +137,10 @@ pub fn download_file(
     hasher: Option<&mut Sha256>,
     notify_handler: &dyn Fn(Notification<'_>),
 ) -> Result<()> {
-    download_file_with_resume(&url, &path, hasher, false, &notify_handler)
+    download_file_with_resume(url, path, hasher, false, &notify_handler)
 }
 
-pub fn download_file_with_resume(
+pub(crate) fn download_file_with_resume(
     url: &Url,
     path: &Path,
     hasher: Option<&mut Sha256>,
@@ -152,6 +151,9 @@ pub fn download_file_with_resume(
     match download_file_(url, path, hasher, resume_from_partial, notify_handler) {
         Ok(_) => Ok(()),
         Err(e) => {
+            if e.downcast_ref::<std::io::Error>().is_some() {
+                return Err(e);
+            }
             let is_client_error = match e.downcast_ref::<DEK>() {
                 // Specifically treat the bad partial range error as not our
                 // fault in case it was something odd which happened.
@@ -247,19 +249,11 @@ fn download_file_(
     res
 }
 
-pub fn parse_url(url: &str) -> Result<Url> {
-    Url::parse(url).with_context(|| format!("failed to parse url: {}", url))
+pub(crate) fn parse_url(url: &str) -> Result<Url> {
+    Url::parse(url).with_context(|| format!("failed to parse url: {url}"))
 }
 
-pub fn cmd_status(name: &'static str, cmd: &mut Command) -> Result<()> {
-    use std::ffi::OsString;
-
-    raw::cmd_status(cmd).with_context(|| RustupError::RunningCommand {
-        name: OsString::from(name),
-    })
-}
-
-pub fn assert_is_file(path: &Path) -> Result<()> {
+pub(crate) fn assert_is_file(path: &Path) -> Result<()> {
     if !is_file(path) {
         Err(anyhow!(format!("not a file: '{}'", path.display())))
     } else {
@@ -267,7 +261,7 @@ pub fn assert_is_file(path: &Path) -> Result<()> {
     }
 }
 
-pub fn assert_is_directory(path: &Path) -> Result<()> {
+pub(crate) fn assert_is_directory(path: &Path) -> Result<()> {
     if !is_directory(path) {
         Err(anyhow!(format!("not a directory: '{}'", path.display())))
     } else {
@@ -275,7 +269,11 @@ pub fn assert_is_directory(path: &Path) -> Result<()> {
     }
 }
 
-pub fn symlink_dir<'a, N>(src: &'a Path, dest: &'a Path, notify_handler: &dyn Fn(N)) -> Result<()>
+pub(crate) fn symlink_dir<'a, N>(
+    src: &'a Path,
+    dest: &'a Path,
+    notify_handler: &dyn Fn(N),
+) -> Result<()>
 where
     N: From<Notification<'a>>,
 {
@@ -289,8 +287,16 @@ where
     })
 }
 
-pub fn hard_or_symlink_file(src: &Path, dest: &Path) -> Result<()> {
-    if hardlink_file(src, dest).is_err() {
+pub(crate) fn hard_or_symlink_file(src: &Path, dest: &Path) -> Result<()> {
+    // Some mac filesystems can do hardlinks to symlinks, some can't.
+    // See rust-lang/rustup#3136 for why it's better never to use them.
+    #[cfg(target_os = "macos")]
+    let force_symlink = fs::symlink_metadata(src)
+        .map(|m| m.file_type().is_symlink())
+        .unwrap_or(false);
+    #[cfg(not(target_os = "macos"))]
+    let force_symlink = false;
+    if force_symlink || hardlink_file(src, dest).is_err() {
         symlink_file(src, dest)?;
     }
     Ok(())
@@ -304,7 +310,7 @@ pub fn hardlink_file(src: &Path, dest: &Path) -> Result<()> {
 }
 
 #[cfg(unix)]
-pub fn symlink_file(src: &Path, dest: &Path) -> Result<()> {
+fn symlink_file(src: &Path, dest: &Path) -> Result<()> {
     std::os::unix::fs::symlink(src, dest).with_context(|| RustupError::LinkingFile {
         src: PathBuf::from(src),
         dest: PathBuf::from(dest),
@@ -312,7 +318,7 @@ pub fn symlink_file(src: &Path, dest: &Path) -> Result<()> {
 }
 
 #[cfg(windows)]
-pub fn symlink_file(src: &Path, dest: &Path) -> Result<()> {
+fn symlink_file(src: &Path, dest: &Path) -> Result<()> {
     // we are supposed to not use symlink on windows
     Err(anyhow!(RustupError::LinkingFile {
         src: PathBuf::from(src),
@@ -320,7 +326,11 @@ pub fn symlink_file(src: &Path, dest: &Path) -> Result<()> {
     }))
 }
 
-pub fn copy_dir<'a, N>(src: &'a Path, dest: &'a Path, notify_handler: &dyn Fn(N)) -> Result<()>
+pub(crate) fn copy_dir<'a, N>(
+    src: &'a Path,
+    dest: &'a Path,
+    notify_handler: &dyn Fn(N),
+) -> Result<()>
 where
     N: From<Notification<'a>>,
 {
@@ -334,13 +344,13 @@ where
     })
 }
 
-pub fn copy_file(src: &Path, dest: &Path) -> Result<()> {
+pub(crate) fn copy_file(src: &Path, dest: &Path) -> Result<()> {
     let metadata = fs::symlink_metadata(src).with_context(|| RustupError::ReadingFile {
         name: "metadata for",
         path: PathBuf::from(src),
     })?;
     if metadata.file_type().is_symlink() {
-        symlink_file(&src, dest).map(|_| ())
+        symlink_file(src, dest).map(|_| ())
     } else {
         fs::copy(src, dest)
             .with_context(|| {
@@ -354,7 +364,7 @@ pub fn copy_file(src: &Path, dest: &Path) -> Result<()> {
     }
 }
 
-pub fn remove_dir<'a, N>(
+pub(crate) fn remove_dir<'a, N>(
     name: &'static str,
     path: &'a Path,
     notify_handler: &dyn Fn(N),
@@ -391,12 +401,10 @@ pub fn remove_file(name: &'static str, path: &Path) -> Result<()> {
     })
 }
 
-pub fn ensure_file_removed(name: &'static str, path: &Path) -> Result<()> {
+pub(crate) fn ensure_file_removed(name: &'static str, path: &Path) -> Result<()> {
     let result = remove_file(name, path);
     if let Err(err) = &result {
-        if let Some(retry::Error::Operation { error: e, .. }) =
-            err.downcast_ref::<retry::Error<io::Error>>()
-        {
+        if let Some(retry::Error { error: e, .. }) = err.downcast_ref::<retry::Error<io::Error>>() {
             if e.kind() == io::ErrorKind::NotFound {
                 return Ok(());
             }
@@ -408,18 +416,19 @@ pub fn ensure_file_removed(name: &'static str, path: &Path) -> Result<()> {
     })
 }
 
-pub fn read_dir(name: &'static str, path: &Path) -> Result<fs::ReadDir> {
+pub(crate) fn read_dir(name: &'static str, path: &Path) -> Result<fs::ReadDir> {
     fs::read_dir(path).with_context(|| RustupError::ReadingDirectory {
         name,
         path: PathBuf::from(path),
     })
 }
 
-pub fn open_browser(path: &Path) -> Result<()> {
-    opener::open(path).context("couldn't open browser")
+pub(crate) fn open_browser(path: &Path) -> Result<()> {
+    opener::open_browser(path).context("couldn't open browser")
 }
 
-pub fn set_permissions(path: &Path, perms: fs::Permissions) -> Result<()> {
+#[cfg(not(windows))]
+fn set_permissions(path: &Path, perms: fs::Permissions) -> Result<()> {
     fs::set_permissions(path, perms).map_err(|e| {
         RustupError::SettingPermissions {
             p: PathBuf::from(path),
@@ -438,7 +447,7 @@ pub fn file_size(path: &Path) -> Result<u64> {
         .len())
 }
 
-pub fn make_executable(path: &Path) -> Result<()> {
+pub(crate) fn make_executable(path: &Path) -> Result<()> {
     #[allow(clippy::unnecessary_wraps)]
     #[cfg(windows)]
     fn inner(_: &Path) -> Result<()> {
@@ -478,23 +487,23 @@ pub fn current_exe() -> Result<PathBuf> {
     env::current_exe().context(RustupError::LocatingWorkingDir)
 }
 
-pub fn to_absolute<P: AsRef<Path>>(path: P) -> Result<PathBuf> {
+pub(crate) fn to_absolute<P: AsRef<Path>>(path: P) -> Result<PathBuf> {
     current_dir().map(|mut v| {
         v.push(path);
         v
     })
 }
 
-pub fn home_dir() -> Option<PathBuf> {
-    home::home_dir_from(&home_process())
+pub(crate) fn home_dir() -> Option<PathBuf> {
+    home::home_dir_with_env(&home_process())
 }
 
-pub fn cargo_home() -> Result<PathBuf> {
-    home::cargo_home_from(&home_process()).context("failed to determine cargo home")
+pub(crate) fn cargo_home() -> Result<PathBuf> {
+    home::cargo_home_with_env(&home_process()).context("failed to determine cargo home")
 }
 
 // Creates a ~/.rustup folder
-pub fn create_rustup_home() -> Result<()> {
+pub(crate) fn create_rustup_home() -> Result<()> {
     // If RUSTUP_HOME is set then don't make any assumptions about where it's
     // ok to put ~/.rustup
     if process().var_os("RUSTUP_HOME").is_some() {
@@ -502,7 +511,7 @@ pub fn create_rustup_home() -> Result<()> {
     }
 
     let home = rustup_home_in_user_dir()?;
-    fs::create_dir_all(&home).context("unable to create ~/.rustup")?;
+    fs::create_dir_all(home).context("unable to create ~/.rustup")?;
 
     Ok(())
 }
@@ -511,16 +520,16 @@ fn dot_dir(name: &str) -> Option<PathBuf> {
     home_dir().map(|p| p.join(name))
 }
 
-pub fn rustup_home_in_user_dir() -> Result<PathBuf> {
+fn rustup_home_in_user_dir() -> Result<PathBuf> {
     // XXX: This error message seems wrong/bogus.
     dot_dir(".rustup").ok_or_else(|| anyhow::anyhow!("couldn't find value of RUSTUP_HOME"))
 }
 
-pub fn rustup_home() -> Result<PathBuf> {
-    home::rustup_home_from(&home_process()).context("failed to determine rustup home dir")
+pub(crate) fn rustup_home() -> Result<PathBuf> {
+    home::rustup_home_with_env(&home_process()).context("failed to determine rustup home dir")
 }
 
-pub fn format_path_for_display(path: &str) -> String {
+pub(crate) fn format_path_for_display(path: &str) -> String {
     let unc_present = path.find(r"\\?\");
 
     match unc_present {
@@ -529,40 +538,6 @@ pub fn format_path_for_display(path: &str) -> String {
     }
 }
 
-pub fn toolchain_sort<T: AsRef<str>>(v: &mut Vec<T>) {
-    use semver::{Identifier, Version};
-
-    fn special_version(ord: u64, s: &str) -> Version {
-        Version {
-            major: 0,
-            minor: 0,
-            patch: 0,
-            pre: vec![Identifier::Numeric(ord), Identifier::AlphaNumeric(s.into())],
-            build: vec![],
-        }
-    }
-
-    fn toolchain_sort_key(s: &str) -> Version {
-        if s.starts_with("stable") {
-            special_version(0, s)
-        } else if s.starts_with("beta") {
-            special_version(1, s)
-        } else if s.starts_with("nightly") {
-            special_version(2, s)
-        } else {
-            Version::parse(&s.replace("_", "-")).unwrap_or_else(|_| special_version(3, s))
-        }
-    }
-
-    v.sort_by(|a, b| {
-        let a_str: &str = a.as_ref();
-        let b_str: &str = b.as_ref();
-        let a_key = toolchain_sort_key(a_str);
-        let b_key = toolchain_sort_key(b_str);
-        a_key.cmp(&b_key)
-    });
-}
-
 #[cfg(target_os = "linux")]
 fn copy_and_delete<'a, N>(
     name: &'static str,
@@ -609,13 +584,12 @@ where
             Ok(()) => OperationResult::Ok(()),
             Err(e) => match e.kind() {
                 io::ErrorKind::PermissionDenied => {
-                    notify_handler(Notification::RenameInUse(&src, &dest).into());
+                    notify_handler(Notification::RenameInUse(src, dest).into());
                     OperationResult::Retry(e)
                 }
                 #[cfg(target_os = "linux")]
-                io::ErrorKind::Other
-                    if process().var_os("RUSTUP_PERMIT_COPY_RENAME").is_some()
-                        && Some(EXDEV) == e.raw_os_error() =>
+                _ if process().var_os("RUSTUP_PERMIT_COPY_RENAME").is_some()
+                    && Some(EXDEV) == e.raw_os_error() =>
                 {
                     match copy_and_delete(name, src, dest, notify_handler) {
                         Ok(()) => OperationResult::Ok(()),
@@ -636,7 +610,7 @@ where
     })
 }
 
-pub fn delete_dir_contents(dir_path: &Path) {
+pub(crate) fn delete_dir_contents(dir_path: &Path) {
     match remove_dir_all::remove_dir_contents(dir_path) {
         Err(e) if e.kind() != io::ErrorKind::NotFound => {
             panic!("Unable to clean up {}: {:?}", dir_path.display(), e);
@@ -645,7 +619,7 @@ pub fn delete_dir_contents(dir_path: &Path) {
     }
 }
 
-pub struct FileReaderWithProgress<'a> {
+pub(crate) struct FileReaderWithProgress<'a> {
     fh: io::BufReader<File>,
     notify_handler: &'a dyn Fn(Notification<'_>),
     nbytes: u64,
@@ -653,7 +627,10 @@ pub struct FileReaderWithProgress<'a> {
 }
 
 impl<'a> FileReaderWithProgress<'a> {
-    pub fn new_file(path: &Path, notify_handler: &'a dyn Fn(Notification<'_>)) -> Result<Self> {
+    pub(crate) fn new_file(
+        path: &Path,
+        notify_handler: &'a dyn Fn(Notification<'_>),
+    ) -> Result<Self> {
         let fh = match File::open(path) {
             Ok(fh) => fh,
             Err(_) => {
@@ -699,7 +676,7 @@ impl<'a> io::Read for FileReaderWithProgress<'a> {
 
 // search user database to get home dir of euid user
 #[cfg(unix)]
-pub fn home_dir_from_passwd() -> Option<PathBuf> {
+pub(crate) fn home_dir_from_passwd() -> Option<PathBuf> {
     use std::ffi::{CStr, OsString};
     use std::mem::MaybeUninit;
     use std::os::unix::ffi::OsStringExt;
@@ -732,35 +709,10 @@ pub fn home_dir_from_passwd() -> Option<PathBuf> {
 
 #[cfg(test)]
 mod tests {
+    use rustup_macros::unit_test as test;
+
     use super::*;
 
-    #[test]
-    fn test_toolchain_sort() {
-        let expected = vec![
-            "stable-x86_64-unknown-linux-gnu",
-            "beta-x86_64-unknown-linux-gnu",
-            "nightly-x86_64-unknown-linux-gnu",
-            "1.0.0-x86_64-unknown-linux-gnu",
-            "1.2.0-x86_64-unknown-linux-gnu",
-            "1.8.0-x86_64-unknown-linux-gnu",
-            "1.10.0-x86_64-unknown-linux-gnu",
-        ];
-
-        let mut v = vec![
-            "1.8.0-x86_64-unknown-linux-gnu",
-            "1.0.0-x86_64-unknown-linux-gnu",
-            "nightly-x86_64-unknown-linux-gnu",
-            "stable-x86_64-unknown-linux-gnu",
-            "1.10.0-x86_64-unknown-linux-gnu",
-            "beta-x86_64-unknown-linux-gnu",
-            "1.2.0-x86_64-unknown-linux-gnu",
-        ];
-
-        toolchain_sort(&mut v);
-
-        assert_eq!(expected, v);
-    }
-
     #[test]
     fn test_remove_file() {
         let tempdir = tempfile::Builder::new().prefix("rustup").tempdir().unwrap();
